package de.andrena.et2016.check.conventions;

import static org.hamcrest.Matchers.hasSize;
import static org.junit.Assert.assertThat;

import java.util.Set;

import org.junit.Test;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.core.type.filter.TypeFilter;

public class ClasspathScannerTest {

	private static final String SCAN_PACKAGE = "de.andrena.et2016.check.demo";

	@Test
	public void checkAllAbstractClassesArePrefixedWithAbstract() {
		Set<BeanDefinition> beans = ClasspathScanner.including(abstractClasses()).exclude(typesNamed("Abstract.*")).scan(SCAN_PACKAGE);
		flush(beans);
		assertThat(beans, hasSize(0));
	}

	@Test
	public void checkAllInterfacesStartWithI() {
		Set<BeanDefinition> beans = ClasspathScanner.including(interfaces()).exclude(typesNamed("I.+")).scan(SCAN_PACKAGE);
		flush(beans);
		assertThat(beans, hasSize(0));
	}

	@Test
	public void checkAllUtilClassesAreDeclaredFinal() {
		Set<BeanDefinition> beans = ClasspathScanner.including(typesNamed(".*Util(s)?")).exclude(finalClasses()).scan(SCAN_PACKAGE);
		flush(beans);
		assertThat(beans, hasSize(0));
	}

	private void flush(Set<BeanDefinition> beans) {
		beans.stream().map(BeanDefinition::getBeanClassName).forEach(System.out::println);
		System.out.println();
	}

	private static TypeFilter abstractClasses() {
		return (metadataReader, mrFactory) -> metadataReader.getClassMetadata().isAbstract() && !metadataReader.getClassMetadata().isInterface();
	}

	private static TypeFilter finalClasses() {
		return (metadataReader, mrFactory) -> metadataReader.getClassMetadata().isFinal();
	}

	private static TypeFilter interfaces() {
		return (metadataReader, mrFactory) -> metadataReader.getClassMetadata().isInterface();
	}

	private static TypeFilter typesNamed(String namePattern) {
		return (metadataReader, mrFactory) -> metadataReader.getClassMetadata().getClassName().matches("(\\S+\\.)*" + namePattern + "$");
	}

}
