package de.andrena.et2016.check.conventions;

import static org.hamcrest.Matchers.is;
import static org.junit.Assert.assertThat;

import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.Collections;
import java.util.Properties;

import org.hamcrest.Description;
import org.hamcrest.Matcher;
import org.hamcrest.TypeSafeDiagnosingMatcher;
import org.junit.Test;

import com.google.common.base.Joiner;

public class PropertiesCheckTest {

	@Test
	public void checkConflictFreeProperties() throws IOException {
		Properties propertiesModule1 = loadPropertiesFrom("properties/module1.properties");
		Properties propertiesModule2 = loadPropertiesFrom("properties/module2.properties");

		assertThat(propertiesModule1.keySet(), is(disjointTo(propertiesModule2.keySet())));
	}

	private Matcher<Collection<?>> disjointTo(Collection<?> otherCollection) {
		return new TypeSafeDiagnosingMatcher<Collection<?>>() {

			@Override
			public void describeTo(Description description) {
				description.appendText("collection disjoint to " + asText(otherCollection));
			}

			@Override
			protected boolean matchesSafely(Collection<?> item, Description mismatchDescription) {
				if (Collections.disjoint(item, otherCollection)) {
					return true;
				}
				mismatchDescription.appendText(asText(item));
				mismatchDescription.appendText(" is not disjoint to ");
				mismatchDescription.appendText(asText(otherCollection));
				return false;
			}
		};
	}

	private static String asText(Collection<?> otherCollection) {
		return "[" + Joiner.on(',').join(otherCollection) + "]";
	}

	private Properties loadPropertiesFrom(String name) throws IOException {
		Properties properties = new Properties();
		InputStream propertiesStream = getClass().getClassLoader().getResourceAsStream(name);
		if (propertiesStream != null) {
			properties.load(propertiesStream);
		}
		return properties;
	}
}
